#include "ShaderConstants.fxh"
#include "util.fxh"
#include "chunk.yss"


struct PS_Input
{
	
    float3 worldpos : WPOS;
	float4 position : SV_Position;
    float3 chunkpos : POSIT;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};


 
ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{



#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
 
#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
	//diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif
 
#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

if(PSInput.color.g!=PSInput.color.r)diffuse.rgb *= PSInput.color.rgb;
else{diffuse.rgb*=lerp(1.0,0.75,smoothstep(0.7,0.3,PSInput.color.g));}
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

float4 text =TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);

bool WATER = PSInput.color.b*2.0>PSInput.color.r+PSInput.color.g&&PSInput.color.a!=0.0;
bool MAT = text.a>0.975&&text.a<0.985;
bool HELL = FOG_COLOR.b<FOG_COLOR.r&&FOG_COLOR.b<0.05&&FOG_COLOR.r<0.5;
bool UWATER = (FOG_COLOR.b>0.15&&(FOG_COLOR.b>FOG_COLOR.g*1.2)&&(FOG_COLOR.b>=FOG_COLOR.r*1.8)&&(FOG_COLOR.g>FOG_COLOR.r*1.6))||(FOG_COLOR.g*1.2>=FOG_COLOR.b)&&(FOG_COLOR.r*1.3<FOG_COLOR.g);


float4 light =TEXTURE_1.Sample(TextureSampler1, float2(0.0,1.0));
float timeuv1=smoothstep(0.462745,0.989235,light.r)*2.0-1.0;
float3 sunposition=SP(timeuv1);

float3 normal = -normalize(cross(ddx(PSInput.worldpos),ddy(PSInput.worldpos)));

if(WATER&&normal.y>0.8)normal=getnormal(PSInput.worldpos,0.2);
if(WATER&&normal.y>0.8&&NORMALSAMPLER)normal=normalize(TEXTURE_0.Sample(TextureSampler0, PSInput.uv0).rbg*2.0-1.0);



float3 reflectposition=normalize(reflect(PSInput.worldpos,normal));
float3 reflectdirection=reflectposition;
reflectposition*=(3000.0-PSInput.worldpos.y)/reflectposition.y;
reflectposition+=PSInput.worldpos;
reflectposition=normalize(reflectposition);

if(HELL){
#include "render/chunkhell.render"
 }
 else {
#include "render/chunkbasic.render"
 }


 






#ifdef FOG
	//diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}